use crate::prelude::*;
use anyhow::Result;
use utilities::prelude::*;

use super::gridinfo::GridInfo;
use super::mandatory::Mandatory;

use std::str::from_utf8;
use std::{
    convert::TryFrom,
    sync::{Arc, RwLock, Weak},
};

use super::validator::{
    cow_to_fill_type, cow_to_grow_direction, cow_to_str, cow_to_text_alignment, str_into,
};

pub struct ProgressBarInfo {
    pub id: String,

    pub x_slot: Mandatory<usize>,
    pub y_slot: Mandatory<usize>,

    pub x_dim: u32,
    pub y_dim: u32,

    pub foreground: Option<FillTypeInfo>,
    pub background: Option<FillTypeInfo>,
    pub direction: Option<GrowDirection>,

    pub text_ratio: Option<f32>,
    pub text: RwLock<String>,
    pub text_color: Color,
    pub text_alignment: TextAlignment,

    pub parent: Weak<GridInfo>,
}

impl ProgressBarInfo {
    pub fn new<'a>(
        attributes: quick_xml::events::attributes::Attributes<'a>,
        grid: &Arc<GridInfo>,
    ) -> Result<ProgressBarInfo> {
        let mut progress_bar_info = ProgressBarInfo {
            id: String::new(),

            x_slot: Mandatory::default(),
            y_slot: Mandatory::default(),

            x_dim: 1,
            y_dim: 1,

            foreground: None,
            background: None,
            direction: None,

            text_ratio: None,
            text: RwLock::new(String::new()),
            text_color: Color::default(),
            text_alignment: TextAlignment::default(),

            parent: Arc::downgrade(grid),
        };

        for attribute_res in attributes {
            let attribute = attribute_res?;

            match attribute.key.into_inner() {
                b"id" => progress_bar_info.id = cow_to_str(attribute.value),
                b"x_slot" => progress_bar_info.x_slot.set(str_into(attribute.value)?),
                b"y_slot" => progress_bar_info.y_slot.set(str_into(attribute.value)?),
                b"x_size" => progress_bar_info.x_dim = str_into(attribute.value)?,
                b"y_size" => progress_bar_info.y_dim = str_into(attribute.value)?,
                b"background" => {
                    progress_bar_info.background = Some(cow_to_fill_type(attribute.value))
                }
                b"foreground" => {
                    progress_bar_info.foreground = Some(cow_to_fill_type(attribute.value))
                }
                b"direction" => {
                    progress_bar_info.direction = Some(cow_to_grow_direction(attribute.value)?)
                }
                b"text_color" => {
                    let text = cow_to_str(attribute.value);
                    progress_bar_info.text_color = Color::try_from(text.as_str())?;
                }
                b"text_ratio" => progress_bar_info.text_ratio = Some(str_into(attribute.value)?),
                b"text_alignment" => {
                    progress_bar_info.text_alignment = cow_to_text_alignment(attribute.value)?
                }
                _ => {
                    return Err(anyhow::Error::msg(format!(
                        "Unsupported attribute in Progress Bar: {}",
                        from_utf8(attribute.key.into_inner())?
                    )))
                }
            }
        }

        Ok(progress_bar_info)
    }
}
